﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/person/%personid%/contact"		'Must start with "/"
	Private header() As String = Array As String("Accept","application/json")
	Private responseHeaderFilter As String = "/eclipse/rest/person/"
	Private failureCount As Long
	Private entityName As String = "Contact"

	Private mapData As Map
	Private mapDoB As Map

	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ( 	"EclipseId", _
																"address", _
																"contactType", _
																"contactUsage", _
																"startDate")
																													
	Private foreignReferenceValue As String
	
	Private jobFilePath As String
	Private jobFileName As String
	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	Private outputFileIsCreated As Boolean = False
	Private identityColumnIdx As Int
	Private idx2 As Int
	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)
	
	mapData.Initialize
	'Set up the data payload for the post
	'Default values are defined here
	'Any value can be provided in the job file to overwrite defaults
	'The fields that must appear in the job file are lists ^^^ in listMandatoryArguments

	resetDataMap
	
	mapDoB.Initialize
	resetMapDoB
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	jobFilePath = fPath
	jobFileName = fName

	
End Sub

Private Sub resetDataMap
	mapData.Clear
	mapData.Put("_type","NewEmailContact")
	mapData.Put("address","")
	mapData.Put("startDate","")
	mapData.Put("contactType","HOME")
	mapData.Put("contactUsage","PERMANENT")
	mapData.Put("preferred","false")
	mapData.Put("objectVersion","0")
	
End Sub

Private Sub resetMapDoB
	mapDoB.Clear
	mapDoB.Put("year","")
	mapDoB.Put("month","")
	mapDoB.Put("day","")
	mapDoB.Put("_type",QUOTE & "SimpleFuzzyDate" & QUOTE)
End Sub

Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
		foreignReferenceValue = listJobFileHdrs.Get(0)
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
			'Mandatory argument does not feature in the column list of the job file
			listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	'Check for _identityColumn
	idx2 = listJobFileHdrs.IndexOf("_IdentityColumn")
	If idx2 = -1 Then
		r.success = False
		r.value = -100
		r.description = "_IdentityColumn not found. This needs to name the column containing the foriegn unique reference"
		r.fatal = True
		Return r
	End If

	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

public Sub execute As ResumableSub
	'Process the job file - row by row
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	
	Dim rc As RestCall
	rc.Initialize

	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file
		Dim rw() As String = listJobFileRows.Get(rowNum)

		identityColumnIdx = listJobFileHdrs.IndexOf(rw(idx2))
		If identityColumnIdx = -1 Then
			r.success = False
			r.value = -100
			r.description = "Column specified in _IdentityColumn not found."
			r.fatal = True
			Return r
		End If
		foreignReferenceValue = rw(identityColumnIdx)
		
		'Populate the data map with values from the job file row
		resetDataMap
		For x = 0 To mapData.Size - 1
			Dim colname As String = mapData.GetKeyAt(x)
			idx = listJobFileHdrs.IndexOf(colname)
			If idx <> -1 Then
				'Column does appear in job file - overide the default
				mapData.Put(mapData.GetKeyAt(x),rw(idx))
			End If
		Next
		
		'Data map is now populated - create a JSON string
		Dim js As StringBuilder
		js.Initialize
		js.Append("{")
		For x = 0 To mapData.Size -1
			Dim colname As String = mapData.GetKeyAt(x)
			Select colname
				Case "startDate"
					'Requires Epoch time
					Dim epoch As Long = Main.utils.dateToEpochTime(mapData.GetValueAt(x))
					js.Append(QUOTE).Append(mapData.GetKeyAt(x)).Append(QUOTE & ":").Append(epoch)
				Case Else
					If mapData.GetValueAt(x) = Null Then
						js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(Null)
					Else
						js.Append(QUOTE).Append(colname).Append(QUOTE & ":" & QUOTE).Append(mapData.GetValueAt(x)).Append(QUOTE)
					End If
			End Select
			If x < mapData.Size -1 Then js.Append(",")
		Next
		js.Append("}")
	
		
		'API Call
		'========
		Dim newApiPath As String = apiPath.Replace("%personid%", foreignReferenceValue)
		Wait For (rc.restPost(newApiPath, header, js.ToString, rowNum)) complete
		
		Dim strAllocatedId As String
		If outputFileIsCreated = False Then
			Main.utils.createOutputFile(listJobFileHdrs.Get(identityColumnIdx),"EclipseId / Error")
			outputFileIsCreated = True
		End If
		If rc.postSuccess = True Then
			If rc.mapResponseHeaders.ContainsKey("location") = False Or rc.httpResponse <> 201 Then
				Log("--Header not received specifying location")
				strAllocatedId = "ERROR - Organisation not created)"
				Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & strAllocatedId & QUOTE)
				Exit
			Else
				Dim listHeaders As List = rc.mapResponseHeaders.Get("location")
				strAllocatedId = listHeaders.Get(0)	'location: /eclipse/rest/organisation/392875
				strAllocatedId = strAllocatedId.Replace(responseHeaderFilter,"")
				'We now have the allocated ID - save this in the results map.
				Log(entityName & " " & strAllocatedId & " created for " & foreignReferenceValue)
			End If
		Else
			strAllocatedId = "ERROR - " & rc.failReason.Replace(QUOTE, "'")
			'Also need to know what errors look like and process them accordingly
			Log("Create FAILED.")
			failureCount = failureCount + 1
		End If

		If outputFileIsCreated = False Then
			Main.utils.createOutputFile(listJobFileHdrs.Get(identityColumnIdx),"EclipseId")
			outputFileIsCreated = True
		End If
		
		Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & strAllocatedId & QUOTE)
	Next		'Next row in job file
	
	Log("===========================")
	If failureCount > 0 Then
		Log("  All done - with " & failureCount & " failures.")
	Else
		Log("  All Successfully done!")
	End If
	Log("===========================")
	ExitApplication2(0)
	Return r		'Just to keep the compiler happy...
	
End Sub