﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.98
@EndOfDesignText@
Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/organisation"
	Private header() As String = Array As String("Accept","application/json")
	Private responseHeaderFilter As String = "/eclipse/rest/organisation/"
	Private failureCount As Long
	Private entityName As String = "OrganisationPlus"
	
	Private mapData As Map
	Private mapAlias As Map
	
'	Private addressStartDate As FuzzyDate

	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ( 	"name", _
																"organisationType", _
																"organisation_startDate", _
																"primaryNameOrNumber", _
																"street", _
																"town", _
																"postcode", _
																"address_type", _
																"address_usage", _
																"address_startDate")

	'"foreignId","name","organisationType","organisation_startDate","organisation_endDate","primaryNameOrNumber",
	'"street","locality","town","county","country",
	'"postcode","address_type","address_usage","address_startDate","systemIdentifier","landline_number",
	'"mobile_number","email_address","_IdentityColumn"
																													
	Private foreignReferenceValue As String
	
	Private jobFilePath As String
	Private jobFileName As String
	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	Private outputFileIsCreated As Boolean = False
	Private identityColumnIdx As Int
	Private idx2 As Int
	

	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)
	mapData.Initialize
	'Set up the data payload for the post
	'Default values are defined here
	'Any value can be provided in the job file to overwrite defaults
	'The fields that must appear in the job file are lists ^^^ in listMandatoryArguments

'	resetDataMap
	
'	addressStartDate.Initialize

	
	'Replacement value names required by the API calls
	mapAlias.Initialize
	mapAlias.Put("landline_number","number")
	mapAlias.Put("mobile_number","number")
	mapAlias.Put("email_address","address")
	mapAlias.Put("address_usage","usage")
	mapAlias.Put("address_type","type")
	mapAlias.Put("address_startDate","startDate")
	mapAlias.Put("organisationType","type")
	mapAlias.Put("organisation_startDate","startDate")
	mapAlias.Put("organisation_endDate","endDate")
	
	
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	jobFilePath = fPath
	jobFileName = fName
End Sub

'Note the field names in the map correspond to the column names in the CSV Job file
'Each field coud have an alias (i.e. the replacement name that the API calls. Thers are defined in mapAlias

Private Sub resetDataMap
	mapData.Clear
	mapData.Put("_type","NewOrganisation")
	mapData.Put("objectVersion",0)
	mapData.Put("name","")
	mapData.Put("type","")
	mapData.Put("subType","")
	mapData.Put("contactVOs","[CONTACT]")		'Indicates one or more contacts from a contact map
	mapData.Put("addresses","[ADDRESS]")		'Indicates one or more addresses from an address map
	mapData.Put("organisation_startDate","")		'Epoch date
	mapData.Put("organisation_endDate","")		'Epoch date	
	mapData.Put("externalResource","{EX_RESOURCE}")	'Indicates values (only one) from another map  SHOULD ALWAYS BE LAST
End Sub





Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
			'Mandatory argument does not feature in the column list of the job file
			listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	'Check for _identityColumn
	idx2 = listJobFileHdrs.IndexOf("_IdentityColumn")
	If idx2 = -1 Then
		r.success = False
		r.value = -100
		r.description = "_IdentityColumn not found. This needs to name the column containing the foriegn unique reference"
		r.fatal = True
		Return r
	End If
	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

public Sub execute As ResumableSub
	'Process the job file - row by row
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	
	Dim rc As RestCall
	rc.Initialize

	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file
		resetDataMap
		Dim rw() As String = listJobFileRows.Get(rowNum)
		
		identityColumnIdx = listJobFileHdrs.IndexOf(rw(idx2))
		If identityColumnIdx = -1 Then
			Log("Column specified in _IdentityColumn not found.")
			ExitApplication2(2)
		End If
		foreignReferenceValue = rw(identityColumnIdx)
		Log("Processing - " & foreignReferenceValue)
		
		'Load the various maps with data from the job file row
		'Special cases are [name] - an array
		'{name} a json object
		For x = 0 To mapData.Size -1
			Dim colname As String = mapData.GetKeyAt(x)
			Dim default As String = mapData.GetValueAt(x)
			idx = listJobFileHdrs.IndexOf(colname)
			If idx <> -1 Then
				'Column DOES appear in job file - overide the default
				If default.StartsWith("{") And default.EndsWith("}") Then
					'This is a json value
					Dim objectName As String = default.Replace("{","").Replace("}","")
					If objectName = "FUZZYDATE" Then
						Dim thisFuzzyDate As FuzzyDate
						thisFuzzyDate.Initialize
						thisFuzzyDate.set(colname, rw, listJobFileHdrs)
						mapData.Put(colname, thisFuzzyDate.toJson(mapAlias))
					End If
					If objectName = "CALCDATE" Then
						Dim thisCalcDate As CalculatedDate
						thisCalcDate.Initialize
						thisCalcDate.set(colname, rw, listJobFileHdrs)
						mapData.Put(colname, thisCalcDate.toJson(mapAlias))
					End If
				Else
					'take the value from the job file
					default = rw(idx)		'Take value as it
					If colname = "organisation_startDate" Or colname = "organisation_endDate" Then
						'Epoch date required
						Dim epoch As Long = Main.utils.dateToEpochTime(default)
						default=epoch
					End If
					mapData.Put(colname, default)
				End If
			Else If default.StartsWith("[") And default.EndsWith("]") Then
				'This is an array type value (DOES NOT appear in the job file)
				Dim objectName As String = default.Replace("[","").Replace("]","")
				Select objectName
					Case "ADDRESS"
						Dim thisAddress As Address
						thisAddress.Initialize
						thisAddress.loadAddress(rw, listJobFileHdrs, mapAlias)
						mapData.Put(colname, "[" & thisAddress.toJson(mapAlias) & "]")	'There will only be 1 address
						
					Case "CONTACT"
						Dim thisLandlineContact As Contact
						thisLandlineContact.Initialize("NewTelephoneContact","LANDLINE")
						thisLandlineContact.loadContact(rw, listJobFileHdrs)

						Dim thisMobileContact As Contact
						thisMobileContact.Initialize("NewTelephoneContact","MOBILE")
						thisMobileContact.loadContact(rw, listJobFileHdrs)

						Dim thisEmailContact As Contact
						thisEmailContact.Initialize("NewEmailContact","")
						thisEmailContact.loadContact(rw, listJobFileHdrs)

						Dim jsonContact As StringBuilder
						jsonContact.Initialize
						If thisLandlineContact.isPresent Then jsonContact.Append(thisLandlineContact.toJson(mapAlias))
						If thisMobileContact.isPresent Then
							If jsonContact.Length > 0 Then jsonContact.Append(",")
							jsonContact.Append(thisMobileContact.toJson(mapAlias))
						End If
						If thisEmailContact.isPresent Then
							If jsonContact.Length > 0 Then jsonContact.Append(",")
							jsonContact.Append(thisEmailContact.toJson(mapAlias))
						End If
						mapData.Put(colname, "[" & jsonContact.ToString & "]")	'There will only be 1 address
				End Select
			Else If default.StartsWith("{") And default.EndsWith("}") Then
				Dim objectName As String = default.Replace("{","").Replace("}","")
				If objectName = "EX_RESOURCE" Then
					'Next level JSON value required
					Dim thisExternalResource As ExternalResource
					thisExternalResource.Initialize(foreignReferenceValue)
					thisExternalResource.loadExternalResource(rw, listJobFileHdrs)
					mapData.Put(mapData.GetKeyAt(x), thisExternalResource.toJson(mapAlias))
					thisExternalResource.dispose
				End If
			End If
		Next


		
		'File.WriteMap(File.DirApp, "CreatePErsonPlusMap.txt", mapData)
			
		'Data map is now populated - swap column names for aliases while creating a json payload
		Dim js As StringBuilder
		js.Initialize
		js.Append("{")
		For x = 0 To mapData.Size -1
			Dim colname As String = mapData.GetKeyAt(x)
			Dim thisValue As String = mapData.GetValueAt(x)
			Select colname
				Case "personTypes"
					'CLIENT is required or a JSON array of > 1 person type
					If thisValue.Contains(",") Then
						'> 1 Person Type provided  e.g. CLIENT,OTHER
						thisValue.Replace(",", QUOTE & "," & QUOTE)
						thisValue = QUOTE & thisValue & QUOTE
					End If
					If thisValue.Contains(QUOTE) = False Then
						thisValue= QUOTE & thisValue & QUOTE
					End If
					If thisValue.StartsWith("[") = False Then
						'JSON array not provided
						thisValue="[" & thisValue & "]"
					End If
					js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(thisValue)
					If x < mapData.Size -1 Then
						js.Append(",")
					End If
					
				Case Else
					If thisValue <> "" Then
						If mapAlias.ContainsKey(colname) Then
							If thisValue.StartsWith("[") Or thisValue.StartsWith("{") Then
								js.Append(QUOTE).Append(mapAlias.Get(colname)).Append(QUOTE & ":").Append(thisValue)
							Else If mapData.GetValueAt(x) = Null Then
								js.Append(QUOTE).Append(mapAlias.Get(colname)).Append(QUOTE & ":").Append(Null)
							Else
								js.Append(QUOTE).Append(mapAlias.Get(colname)).Append(QUOTE & ":" & QUOTE).Append(thisValue).Append(QUOTE)
							End If
						Else
							If thisValue.StartsWith("[") Or thisValue.StartsWith("{") Then
								js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(thisValue)
							Else If mapData.GetValueAt(x) = Null Then
								js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(Null)
							Else
								js.Append(QUOTE).Append(colname).Append(QUOTE & ":" & QUOTE).Append(thisValue).Append(QUOTE)
							End If
						End If
						If x < mapData.Size -1 Then
							js.Append(",")
						End If
					End If
			End Select

		Next
		js.Append("}")
		'File.WriteString(File.DirApp, "CreatePersonPlusPayload.txt",js.ToString)
		
		'API Call
		'========
		
		Wait For (rc.restPost(apiPath, header, js.ToString, rowNum)) complete
		
		Dim strAllocatedId As String
		If outputFileIsCreated = False Then
			Main.utils.createOutputFile(listJobFileHdrs.Get(identityColumnIdx),"EclipseId / Error")
			outputFileIsCreated = True
		End If
		If rc.postSuccess = True Then
			If rc.mapResponseHeaders.ContainsKey("location") = False Or rc.httpResponse <> 201 Then
				Log("--Header not received specifying location")
				strAllocatedId = "ERROR - Organisation not created)"
				Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & strAllocatedId & QUOTE)
				Exit
			Else
				Dim listHeaders As List = rc.mapResponseHeaders.Get("location")
				strAllocatedId = listHeaders.Get(0)	'location: /eclipse/rest/organisation/392875
				strAllocatedId = strAllocatedId.Replace(responseHeaderFilter,"")
				'We now have the allocated ID - save this in the results map.
				Log(entityName & " " & strAllocatedId & " created for " & foreignReferenceValue)
			End If
		Else
			strAllocatedId = "ERROR - " & rc.failReason.Replace(QUOTE, "'")
			'Also need to know what errors look like and process them accordingly
			Log("Create FAILED.")
			failureCount = failureCount + 1
		End If
		
		
		If outputFileIsCreated = False Then
			'Start a new CSV results file
			If File.Exists(Main.gresultsPath, Main.gresultsName) = True Then
				If File.Delete(Main.gresultsPath, Main.gresultsName) = False Then
					Log("Unable to delete previous results file")
					ExitApplication2(2)
				End If
			End If
			Dim strRow As String = QUOTE & listJobFileHdrs.Get(identityColumnIdx) & QUOTE & "," & QUOTE & "EclipseId" & QUOTE
			Main.utils.addToOutputFile(strRow)
			outputFileIsCreated = True
		End If
		
		Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & strAllocatedId & QUOTE)
	Next		'Next row in job file
	
	Log("===========================")
	If failureCount > 0 Then
		Log("  All done - with " & failureCount & " failures.")
	Else
		Log("  All Successfully done!")
	End If
	Log("===========================")
	ExitApplication2(0)
	Return r		'Just to keep the compliter happy...
	
End Sub