﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
'Method used to add location records to Eclipse.
'Received CSV row contains all values to use when creating the location
'The job file row contains the necessary fields to make up the api post payload plus "_IdentityColumn" 
'which contains the name of the column containing the foreign key  to be mapped to the allocated id following the post
'_identityColumn
'UPRN

'Note - This is a 2-stage process
'1) Post the location details and get an address ID
'2) Post the allocated location ID against the person (identified in the EclipseId column of the job file)

Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/location"		'Must start with "/"
	Private apiPath2 As String = "/eclipse/rest/person/%personid%/address"
	Private header() As String = Array As String("Accept","application/json")
	Private responseHeaderFilter1 As String = "/eclipse/rest/location/"
	Private responseHeaderFilter2 As String = "/eclipse/rest/personAddress/"
	Private listValuesWithoutQuotes As List = Array As String("startDate", "objectVersion","personId")		'No quotes in api payload
	Private failureCount As Long
'	Private entityName As String = "Person Address"

	Private mapData As Map
	Private mapData2 As Map
	Private mapDoB As Map

	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ( 	"primaryNameOrNumber", _
	 															"street", _
																"locality", _
																"town", _
																"postcode", _
																"type", _				'Step 2 call
																"usage", _				'Step 2 call
																"startDate", _
																"personId" )
																													
	Private foreignReferenceValue As String
	
	Private jobFilePath As String
	Private jobFileName As String
	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	Private outputFileIsCreated As Boolean = False
	Private identityColumnIdx As Int
	Private idx2 As Int
	
	Private allocatedId As String
	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)
	
	mapData.Initialize
	mapData2.Initialize
	'Set up the data payload for the post
	'Default values are defined here
	'Any value can be provided in the job file to overwrite defaults
	'The fields that must appear in the job file are lists ^^^ in listMandatoryArguments

	resetDataMap
	
	mapDoB.Initialize
	resetMapDoB
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	jobFilePath = fPath
	jobFileName = fName

	
End Sub

Private Sub resetDataMap
	mapData.Clear
	mapData.Put("_type","NewLocation")
	mapData.Put("primaryNameOrNumber",Null)
	mapData.Put("street",Null)
	mapData.Put("locality",Null)
	mapData.Put("town",Null)
	mapData.Put("postcode",Null)
	mapData.Put("countryCode","GBR")
	mapData.Put("countyCode",Null)
	mapData.Put("countrySubdivisionCode",Null)
	mapData.Put("objectVersion","0")
	
	mapData2.Put("_type","NewPersonAddress")
	mapData2.Put("startDate","")	'Note this appears like {"year":9999,"month":99,"day":99,"_type":"SimpleFuzzyDate"}
	mapData2.Put("type","HOME")
	mapData2.Put("usage","PERMANENT")
	mapData2.Put("locationId",Null)		'Id received from call #1
	mapData2.Put("roomDescription",Null)
	mapData2.Put("floorDescription",Null)
	mapData2.Put("unknownLocationType","")
	mapData2.Put("objectVersion","0")
	mapData2.Put("personId",Null)			'Eclipse Id from the job file
	
	
	
End Sub

Private Sub resetMapDoB
	mapDoB.Clear
	mapDoB.Put("_type","SimpleFuzzyDate")
	mapDoB.Put("year",Null)
	mapDoB.Put("month",Null)
	mapDoB.Put("day",Null)
End Sub

Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
		foreignReferenceValue = listJobFileHdrs.Get(0)
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
			'Mandatory argument does not feature in the column list of the job file
			listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	'Check for _identityColumn
	idx2 = listJobFileHdrs.IndexOf("_IdentityColumn")
	If idx2 = -1 Then
		r.success = False
		r.value = -100
		r.description = "_IdentityColumn not found. This needs to name the column containing the foriegn unique reference"
		r.fatal = True
		Return r
	End If

	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

public Sub execute As ResumableSub
	'Process the job file - row by row
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	Dim rc As RestCall
	rc.Initialize
	

	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file
		Dim rw() As String = listJobFileRows.Get(rowNum)

		identityColumnIdx = listJobFileHdrs.IndexOf(rw(idx2))
		If identityColumnIdx = -1 Then
			Log("Column specified in _IdentityColumn not found.")
			ExitApplication2(2)
		End If
		foreignReferenceValue = rw(identityColumnIdx)
		
		'Populate the data map(s) with values from the job file row
		resetDataMap
		resetMapDoB
		'Get items for first API call from the job file
		For x = 0 To mapData.Size - 1
			Dim colname As String = mapData.GetKeyAt(x)
			idx = listJobFileHdrs.IndexOf(colname)
			If idx <> -1 Then
				'Column does appear in job file - overide the default
				mapData.Put(mapData.GetKeyAt(x),rw(idx))				
			End If
		Next
		'Get items for second API call from the job file
		For x = 0 To mapData2.Size - 1
			Dim colname As String = mapData2.GetKeyAt(x)
			idx = listJobFileHdrs.IndexOf(colname)
			If idx <> -1 Then
				Select colname
					Case "startDate"
						'Need to construct the date
						'Note this appears like {"year":9999,"month":99,"day":99,"_type":"SimpleFuzzyDate"}
						resetMapDoB
						Dim DoB As String = rw(idx)		'Expected in the form yyyy-MM-dd
						If DoB.Length = 10 Then
							mapDoB.Put("year",DoB.SubString2(0,4))
							mapDoB.Put("month",DoB.SubString2(5,7))
							mapDoB.Put("day",DoB.SubString(8))
						End If
						'Change the string in the job file
						Dim fd As StringBuilder
						fd.Initialize
						fd.Append("{")
						Dim i As Int
						For i = 0 To mapDoB.Size -1
							fd.Append(QUOTE).Append(mapDoB.GetKeyAt(i)).Append(QUOTE & ":").Append(QUOTE & mapDoB.GetValueAt(i) & QUOTE)
							If i < mapDoB.Size -1 Then fd.Append(",")
						Next
						fd.Append("}")
						mapData2.Put(mapData2.GetKeyAt(x),fd.ToString)
					Case Else
						'Column does appear in job file - overide the default
						mapData2.Put(mapData2.GetKeyAt(x),rw(idx))
				End Select
			End If
		Next
		
		'Data maps are now populated - 
		
		' Build #1
		Log("Call #1 " & apiPath)
		Wait For (apiPostCall(apiPath, mapData, header, rowNum, responseHeaderFilter1, "Location", rc)) complete
		If allocatedId.StartsWith("ERROR") = False Then
			'Call 1 didn't fail
			'We now have the allocated ID from ^^^^ - save this in the results map.
			mapData2.Put("locationId",allocatedId)	'Carry the allocated location ID across

			'Build call #2
			Dim newApiPath2 As String = apiPath2.Replace("%personid%",rw(identityColumnIdx))
			Log("Call #1 " & newApiPath2)
			wait for (apiPostCall(newApiPath2, mapData2, header, rowNum + listJobFileRows.Size, responseHeaderFilter2, "Person Address",rc)) complete
			If allocatedId.StartsWith("ERROR") Then
				'Call failed

			End If
			
		End If
		




	Next		'Next row in job file
	
	Log("===========================")
	If failureCount > 0 Then
		Log("  All done - with " & failureCount & " failures.")
	Else
		Log("  All Successfully done!")
	End If
	Log("===========================")
	ExitApplication2(0)
	Return r		'Just to keep the compiler happy...
	
End Sub

Private Sub apiPostCall(thisApiPath As String, thisMap As Map, thisHeader() As String, rownum As Int, responseMask As String, thisEntityName As String, rest As RestCall ) As ResumableSub
	' create a JSON string for call	
	Dim js As StringBuilder
	js.Initialize
	
	js.Append("{")
	For x = 0 To mapData.Size -1
		Dim colname As String = thisMap.GetKeyAt(x)
		If thisMap.GetValueAt(x) = Null Then
			js.Append(QUOTE).Append(thisMap.GetKeyAt(x)).Append(QUOTE & ":" ).Append(Null)
		Else
			If listValuesWithoutQuotes.IndexOf(colname) <> -1 Then
				js.Append(QUOTE).Append(thisMap.GetKeyAt(x)).Append(QUOTE & ":").Append(thisMap.GetValueAt(x))
			Else
				js.Append(QUOTE).Append(thisMap.GetKeyAt(x)).Append(QUOTE & ":" & QUOTE).Append(thisMap.GetValueAt(x)).Append(QUOTE)
			End If	
		End If
		If x < thisMap.Size -1 Then js.Append(",")
	Next
	js.Append("}")
	
		
	'API Call
	'===========		
	Wait For (rest.restPost(thisApiPath, thisHeader, js.ToString, rownum)) complete
	
	'Response received		
	allocatedId = ""
	If outputFileIsCreated = False Then
		Main.utils.createOutputFile(listJobFileHdrs.Get(identityColumnIdx),"EclipseId / Error")
		outputFileIsCreated = True
	End If
	If rest.postSuccess = True Then
		If rest.mapResponseHeaders.ContainsKey("location") = False Or rest.httpResponse <> 201 Then
			Log("--Header not received specifying location or bad response")
			allocatedId = "ERROR - " & thisEntityName & " not created)"
			Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & allocatedId & QUOTE)
			failureCount = failureCount + 1
		Else
			Dim listHeaders As List = rest.mapResponseHeaders.Get("location")
			allocatedId = listHeaders.Get(0)	'location: 
			allocatedId = allocatedId.Replace(responseMask,"")
			Log(thisEntityName & " " & allocatedId & " created for " & foreignReferenceValue)
		End If
	Else
		allocatedId = "ERROR - " & rest.failReason
		'Also need to know what errors look like and process them accordingly
		Log("Create FAILED.")
		failureCount = failureCount + 1
		If outputFileIsCreated = False Then
			Main.utils.createOutputFile(listJobFileHdrs.Get(identityColumnIdx),"EclipseId")
			outputFileIsCreated = True
		End If
		Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & allocatedId & QUOTE)
	End If
	return True
End Sub

