﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
'Method used to add person records to Eclipse.
'Received CSV row contains all values to use when creating the person
'The job file row contains the necessary fields to make up the api post payload plus "_IdentityColumn" 
'which contains the name of the column containing the foreign key  to be mapped to the allocated id following the post
'_identityColumn
'HospitalNumber


Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/person"		'Must start with "/"
	Private header() As String = Array As String("Accept","application/json")
	Private responseHeaderFilter As String = "/eclipse/rest/person/"
	Private failureCount As Long
	Private entityName As String = "Person"
	
	Private mapData As Map
	Private mapDoB As Map

	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ( 	"forename", _
																"surname", _
																"dateOfBirth", _
																"nhsNumber", _
																"systemIdentifier" )
	Private mapAlias As Map	'Currently unused
																													
	Private foreignReferenceValue As String
	
	Private jobFilePath As String
	Private jobFileName As String
	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	Private outputFileIsCreated As Boolean = False
	Private identityColumnIdx As Int
	Private idx2 As Int
	

	
End Sub

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)
	
	mapData.Initialize
	'Set up the data payload for the post
	'Default values are defined here
	'Any value can be provided in the job file to overwrite defaults
	'The fields that must appear in the job file are lists ^^^ in listMandatoryArguments
	mapAlias.Initialize
	resetDataMap
	
	mapDoB.Initialize
	resetMapDoB
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	jobFilePath = fPath
	jobFileName = fName

	
End Sub

Private Sub resetDataMap
	mapData.Clear
	mapData.Put("_type","NewPerson")
	mapData.Put("title","")
	mapData.Put("forename","")
	mapData.Put("middleNames","")
	mapData.Put("surname","")
	mapData.Put("dateOfBirth","{CALCDATE}")	
	mapData.Put("dateOfBirthEstimated",Null)
	mapData.Put("gender",Null)
	mapData.Put("ethnicity",Null)
	mapData.Put("personTypes","CLIENT")		'Default
	mapData.Put("lifeState","ALIVE")
	mapData.Put("nhsNumber","")
	mapData.Put("chiNumber",Null)
	mapData.Put("professionalTitle","")
	mapData.Put("organisationName","")
	mapData.Put("externalResource","{EX_RESOURCE}")	'Indicates values (only one) from another map  SHOULD ALWAYS BE LAST
End Sub

Private Sub resetMapDoB
	mapDoB.Clear
	mapDoB.Put("year","")
	mapDoB.Put("month","")
	mapDoB.Put("day","")
	mapDoB.Put("_type","SimpleFuzzyDate")	
End Sub

Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
			'Mandatory argument does not feature in the column list of the job file
			listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	'Check for _identityColumn
	idx2 = listJobFileHdrs.IndexOf("_IdentityColumn")
	If idx2 = -1 Then
		r.success = False
		r.value = -100
		r.description = "_IdentityColumn not found. This needs to name the column containing the foriegn unique reference"
		r.fatal = True
		Return r
	End If
	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

public Sub execute As ResumableSub
	'Process the job file - row by row
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	
	Dim rc As RestCall
	rc.Initialize

	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file
		Dim rw() As String = listJobFileRows.Get(rowNum)
		resetDataMap
		
		identityColumnIdx = listJobFileHdrs.IndexOf(rw(idx2))
		If identityColumnIdx = -1 Then
			r.success = False
			r.value = -100
			r.description = "Column specified in _IdentityColumn not found."
			r.fatal = True
			Return r
		End If
		foreignReferenceValue = rw(identityColumnIdx)
		
		'Populate the data map with values from the job file row
		For x = 0 To mapData.Size - 1
			Dim colname As String = mapData.GetKeyAt(x)
			Dim value As String = mapData.GetValueAt(x)
			idx = listJobFileHdrs.IndexOf(colname)
			If idx <> -1 Then
				'Column DOES appear in job file - overide the default
				If value.StartsWith("{") And value.EndsWith("}") Then
					'This is a json value
					Dim objectName As String = value.Replace("{","").Replace("}","")
					If objectName = "FUZZYDATE" Then
						Dim thisFuzzyDate As FuzzyDate
						thisFuzzyDate.Initialize
						thisFuzzyDate.set(colname, rw, listJobFileHdrs)
						mapData.Put(colname, thisFuzzyDate.toJson(mapAlias))
					End If
					If objectName = "CALCDATE" Then
						Dim thisCalcDate As CalculatedDate
						thisCalcDate.Initialize
						thisCalcDate.set(colname, rw, listJobFileHdrs)
						mapData.Put(colname, thisCalcDate.toJson(mapAlias))
					End If
				Else
					mapData.Put(colname, rw(idx))					
				End If
				
			Else
				'Column DOES NOT appear in the job file
				If value.StartsWith("{") And value.EndsWith("}") Then
					Dim objectName As String = value.Replace("{","").Replace("}","")
					If objectName = "EX_RESOURCE" Then
						'Next level JSON value required
						Dim thisExternalResource As ExternalResource
						thisExternalResource.Initialize(foreignReferenceValue)
						thisExternalResource.loadExternalResource(rw, listJobFileHdrs)
						mapData.Put(mapData.GetKeyAt(x), thisExternalResource.toJson(mapAlias))
						thisExternalResource.dispose
					End If
				End If
			End If
		Next
		
		'Data map is now populated - create a JSON string
		Dim js As StringBuilder
		js.Initialize
		js.Append("{")
		For x = 0 To mapData.Size -1
			Dim colname As String = mapData.GetKeyAt(x)
			Dim value As String = mapData.GetValueAt(x)
			If value <> "" Then	'Don't include empty calues
				Select colname

					Case "personTypes"
						'CLIENT is required or a JSON array of > 1 person type
						Dim tp As String = mapData.GetValueAt(x)
						If tp.Contains(",") Then
							'> 1 Person Type provided  e.g. CLIENT,OTHER
							tp.Replace(",", QUOTE & "," & QUOTE)
							tp = QUOTE & tp & QUOTE
						End If
						If tp.Contains(QUOTE) = False Then
							tp= QUOTE & tp & QUOTE
						End If
						If tp.StartsWith("[") = False Then
							'JSON array not provided
							tp="[" & tp & "]"
						End If
						js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(tp)
						
					Case Else
						If mapData.GetValueAt(x) = Null Then
							js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(Null)
						Else If value.StartsWith("{") Then
							js.Append(QUOTE).Append(colname).Append(QUOTE & ":").Append(value)
						Else
							js.Append(QUOTE).Append(colname).Append(QUOTE & ":" & QUOTE).Append(value).Append(QUOTE)
						End If
				End Select
				If x < mapData.Size -1 Then 
					js.Append(",")
				End If
			End If
		Next
		js.Append("}")
	
		'API Call
		'========
		
		Wait For (rc.restPost(apiPath, header, js.ToString, rowNum)) complete
		
		Dim strAllocatedId As String
		If outputFileIsCreated = False Then
			Main.utils.createOutputFile(listJobFileHdrs.Get(identityColumnIdx),"EclipseId / Error")
			outputFileIsCreated = True
		End If
		If rc.postSuccess = True Then
			If rc.mapResponseHeaders.ContainsKey("location") = False Or rc.httpResponse <> 201 Then
				Log("--Header not received specifying location")
				strAllocatedId = "ERROR - Organisation not created)"
				Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & strAllocatedId & QUOTE)
				Exit
			Else
				Dim listHeaders As List = rc.mapResponseHeaders.Get("location")
				strAllocatedId = listHeaders.Get(0)	'location: /eclipse/rest/organisation/392875
				strAllocatedId = strAllocatedId.Replace(responseHeaderFilter,"")
				'We now have the allocated ID - save this in the results map.
				Log(entityName & " " & strAllocatedId & " created for " & foreignReferenceValue)
			End If
		Else
			strAllocatedId = "ERROR - " & rc.failReason.Replace(QUOTE, "'")
			'Also need to know what errors look like and process them accordingly
			Log("Create FAILED.")
			failureCount = failureCount + 1
		End If
		
		
		If outputFileIsCreated = False Then
			'Start a new CSV results file
			If File.Exists(Main.gresultsPath, Main.gresultsName) = True Then
				If File.Delete(Main.gresultsPath, Main.gresultsName) = False Then
					Log("Unable to delete previous results file")
					ExitApplication2(2)
				End If
			End If
			Dim strRow As String = QUOTE & listJobFileHdrs.Get(identityColumnIdx) & QUOTE & "," & QUOTE & "EclipseId" & QUOTE
			Main.utils.addToOutputFile(strRow)
			outputFileIsCreated = True
		End If
		
		Main.utils.addToOutputFile(QUOTE & foreignReferenceValue & QUOTE & "," & QUOTE & strAllocatedId & QUOTE)
	Next		'Next row in job file
	
	Log("===========================")
	If failureCount > 0 Then
		Log("  All done - with " & failureCount & " failures.")
	Else
		Log("  All Successfully done!")
	End If
	Log("===========================")
	ExitApplication2(0)
	Return r		'Just to keep the compliter happy...	
	
End Sub