﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/person/%personid%/address"		'Must start with "/"
	Private header() As String = Array As String()


	Private requiredFixedArgument As String = ""
	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ()	
	
	'These are Optional items that can appear in the job file and are sent to the API
	Private listOptionalArguments As List = Array As String ( )
	Private listValuesToRetrieve As List		'Values for the API responses to return in a new file
	Private listValuesToCarryOver As List		'Values used to map original fields to retrieved values
	Private listExcludeIfEmpty As List
	Private listExcludeIfNotEmpty As List
	
	Private outputFileIsCreated As Boolean
	Private mapArgumentDefaults As Map
														
	Private jobFilePath As String
	Private jobFileName As String

	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	'Output file details
	Private listOutputFileHdrs As List
	Private listOutputFileRows As List
	
	Private previousValuesToRetrieve As String
	Private previousValuesToCarryOver As String
	Private previousExcludeIfEmpty As String
	Private previousExcludeIfNotEmpty As String
	

End Sub


'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)
	'Set up default arguments
	mapArgumentDefaults.Initialize
		
	listValuesToCarryOver.Initialize
	listValuesToRetrieve.Initialize
	listOutputFileHdrs.Initialize
	listOutputFileRows.Initialize
	listExcludeIfEmpty.Initialize
	listExcludeIfNotEmpty.Initialize
	
	jobFilePath = fPath
	jobFileName = fName
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	
	outputFileIsCreated = False		'Indicates if we are appending to the file or not
	
End Sub

Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If mapArgumentDefaults.ContainsKey(listMandatoryArguments.Get(x)) = False Then
			'This mandatory argument doesn't have a default therefore it must exist in the CSV headers
			If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
				'it does not feature in the column list
				listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
			End If
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

Public Sub Execute As ResumableSub
	'Process the job file
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	
	Dim workingFromACohortFile As Boolean
	
	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file
		
		Dim rw() As String = listJobFileRows.Get(rowNum)
		Dim listCohort As List
		listCohort.Initialize
		
		'Check if the mandatory id field contains an actual ID or a COHORT(<filename>) indicator
		'A cohort is a single list of ids
		Dim idColName As String = "personid"
		idx = listJobFileHdrs.IndexOf(idColName)
		If idx = -1 Then
			Main.logResult("Required column '" & idColName & "' not found in job file")
			ExitApplication2(2)
		End If
		If rw(idx).StartsWith("COHORT(") Then
			'Indirect cohort file is specified
			Dim fileCohort As String = rw(idx).SubString(7)
			fileCohort = fileCohort.SubString2(0, fileCohort.Length -1)
			Dim fpCohort As String = File.GetFileParent(fileCohort)
			Dim fnCohort As String = File.GetName(fileCohort)
			If File.Exists(fpCohort, fnCohort) = False Then
				Main.logResult("Cohort file " & fpCohort & "/" & fnCohort & " not found")
				ExitApplication2(2)
			End If
			Dim cohortRows As List
			Dim cohortHdrs As List
			Try
				cohortRows = Main.su.LoadCSV2(fpCohort, fnCohort,",",cohortHdrs)
			Catch
				Main.logResult("Unable to read Cohort file " & fpCohort & "/" & fnCohort & " It needs to be a single column CSV file")
				ExitApplication2(2)
			End Try
			For x = 0 To cohortRows.Size -1
				Dim crw() As String = cohortRows.Get(x)
				If crw.Length = 1 Then
					listCohort.Add(crw(0))
				End If
			Next
			workingFromACohortFile = True
		Else
			listCohort.Add(rw(idx))		'A real value in the id column
			workingFromACohortFile = False
		End If
		
		'Remove any empty items from the cohort
		For x = listCohort.Size -1 To 0 Step -1
			If listCohort.Get(x) = "" Then listCohort.RemoveAt(x)
		Next
		
		Dim cohortItem As Long
		For cohortItem = 0 To listCohort.Size -1		'For each cohort value
					
			'Construct a payload MAP containing values from the job file, the default list and the special values list
			Dim mapCall As Map		'Contains all of the elements for the API call as Key,Value pairs
			mapCall.Initialize
			

			'Grab the mandatory items
			For x = 0 To listMandatoryArguments.Size -1
				idx = listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x))
				If idx <> -1 Then
					'Add this to the argument list
					Dim strVal As String = rw(idx)
					If strVal = "" Then
						'No value provided in job file, its it one of our defaults - substitute it
						If mapArgumentDefaults.ContainsKey(listMandatoryArguments.Get(x)) Then
							strVal = mapArgumentDefaults.Get(listMandatoryArguments.Get(x))
						End If
					End If
					mapCall.Put(listMandatoryArguments.Get(x), strVal)
				End If
			Next		'Next mandatory argument
			
			'Grab the non-mandatory (Optional) argument values from the job file
			For x = 0 To listOptionalArguments.Size -1
				idx = listJobFileHdrs.IndexOf(listOptionalArguments.Get(x))
				If idx <> -1 Then
					'This optional argument appears in the job file - include it
					If mapCall.ContainsKey(listOptionalArguments.Get(x)) = False Then
						mapCall.Put(listOptionalArguments.Get(x), rw(idx))
					End If
				End If
			Next		'Next optional argument
					
			'Get the list of values to retrieve from responses
			idx = listJobFileHdrs.IndexOf("VALUES_TO_RETRIEVE")
			If idx <> -1 Then
				If rw(idx) = "" Then
					rw(idx) = previousValuesToRetrieve
				Else
					previousValuesToRetrieve = rw(idx)
				End If
				listValuesToRetrieve = Main.sf.Split(rw(idx),",")
			End If
			
			'Get the list of values to carry over
			idx = listJobFileHdrs.IndexOf("VALUES_TO_CARRY_OVER")
			If idx <> -1 Then
				If rw(idx) = "" Then
					rw(idx) = previousValuesToCarryOver
				Else
					previousValuesToCarryOver = rw(idx)
				End If
				listValuesToCarryOver = Main.sf.Split(rw(idx),",")
			End If
			
			'Check the carry over values actually exist in the job file
			If listValuesToCarryOver.Size > 0 Then
				For x = (listValuesToCarryOver.Size -1) To 0 Step -1
					If listValuesToCarryOver.Get(x) <> "" Then
						idx = listJobFileHdrs.IndexOf(listValuesToCarryOver.Get(x))
						If idx = -1 Then
							'Doesn't exist - lose it
							listValuesToCarryOver.RemoveAt(x)
						End If
					End If
				Next
			End If
			
			'Get the list of values to check to exclude from output if empty
			idx = listJobFileHdrs.IndexOf("EXCLUDE_IF_EMPTY")
			If idx <> -1 Then
				If rw(idx) = "" Then
					rw(idx) = previousExcludeIfEmpty
				Else
					previousExcludeIfEmpty = rw(idx)
				End If
				listExcludeIfEmpty = Main.sf.Split(rw(idx),",")
			End If
			
			'Get the list of values to check to exclude from output if NOT empty
			idx = listJobFileHdrs.IndexOf("EXCLUDE_IF_NOT_EMPTY")
			If idx <> -1 Then
				If rw(idx) = "" Then
					rw(idx) = previousExcludeIfNotEmpty
				Else
					previousExcludeIfNotEmpty = rw(idx)
				End If
				listExcludeIfNotEmpty = Main.sf.Split(rw(idx),",")
			End If

			
			'Create header columns for the output file
			listOutputFileHdrs.Clear
			listOutputFileHdrs.AddAll(listValuesToCarryOver)
			listOutputFileHdrs.AddAll(listValuesToRetrieve)
			'strip any empty names that came across
			For x = listOutputFileHdrs.Size -1 To 0 Step -1
				If listOutputFileHdrs.Get(x) = "" Then listOutputFileHdrs.RemoveAt(x)
			Next
			
			'Construct the API call
			Dim rc As RestCall
			rc.Initialize
				
			' --------------------------------------------------------
			'	A P I   C A L L  
			' --------------------------------------------------------
			'Construct the payload for the API call
			
			'Create a payload from the map
			Dim sb As StringBuilder
			sb.Initialize
			For x = 0 To mapCall.Size -1
				If x > 0 Then sb.Append("&")
				sb.Append(mapCall.GetKeyAt(x)).Append("=").Append(mapCall.GetValueAt(x))
			Next

			Dim payload As String
			If requiredFixedArgument <> "" Then payload = requiredFixedArgument & "&" & sb.ToString
						
			'Make the API call
			'=================
			Dim newApiPath As String = apiPath.Replace("%personid%", listCohort.Get(cohortItem))		'Substitute the variable part of the path
			Wait For (rc.restGet(newApiPath, header, payload, cohortItem)) Complete


			Dim listResults As List
			listResults = responseParser(rc.strResponse, listValuesToRetrieve)
			
			'We now have a list of maps. Each map containing a set of requested return values
			
			If outputFileIsCreated = False Then
				'Start a new CSV results file
				If File.Exists(Main.gresultsPath, Main.gresultsName) = True Then
					If File.Delete(Main.gresultsPath, Main.gresultsName) = False Then
						Log("Unable to delete previous results file")
						ExitApplication2(2)
					End If
				End If
				Dim sb As StringBuilder
				sb.Initialize
				For x = 0 To listOutputFileHdrs.Size -1
					If sb.Length > 1 Then sb.Append(",")
					sb.Append(QUOTE).Append(listOutputFileHdrs.Get(x)).Append(QUOTE)
				Next
				Main.utils.addToOutputFile(sb.ToString)
				outputFileIsCreated = True
			End If
						
			
			'Results now contained  (each one a map containing required values)
			For x = 0 To listResults.Size -1	'For each result

				Dim mapJ As Map = listResults.Get(x)
				
				Dim includeThisRow As Boolean = True
				Dim col As Int
				Dim sb As StringBuilder
				sb.Initialize

				For col = 0 To listOutputFileHdrs.Size -1	'For each column
					Dim strCol As String
					If mapJ.ContainsKey(listOutputFileHdrs.Get(col)) Then
						'This column comes from the API response
						
						'Check if this column is to be excluded if empty
						strCol = mapJ.Get(listOutputFileHdrs.Get(col))
						strCol = Main.utils.deNull(strCol)
						If listExcludeIfEmpty.IndexOf(listOutputFileHdrs.Get(col)) <> -1 Then
							'Exclude this column if empty
							If strCol = "" Then
								includeThisRow = False
								Exit
							End If
						End If
						'Check if this column is to be excluded if NOT empty
						If listExcludeIfNotEmpty.IndexOf(listOutputFileHdrs.Get(col)) <> -1 Then
							'Exclude this column if NOT empty
							If strCol <> "" Then
								includeThisRow = False
								Exit
							End If
						End If
					Else
						idx = listValuesToCarryOver.IndexOf(listOutputFileHdrs.Get(col)) 
						If idx <> -1 Then
							'This column is one of the valiues we are asked to carry across from the job file
							If workingFromACohortFile Then
								If listOutputFileHdrs.Get(col) = "personid" Then
									strCol = listCohort.Get(cohortItem)
								Else
									strCol = rw(idx)
								End If
							Else
								Dim idx2 As Int = listJobFileHdrs.IndexOf(listOutputFileHdrs.Get(col))
								If idx2 <> -1 Then
									strCol = rw(idx2)
								Else
									strCol = "WRONG VALUE_TO_CARRY_OVER"
								End If
							End If
						Else
							strCol = "WRONG COLUMN NAME"
						End If
					End If
					
					If sb.Length > 0 Then sb.Append(",")
					sb.Append(QUOTE).Append(strCol).Append(QUOTE)
				Next		'Next column
				
				If includeThisRow Then Main.utils.addToOutputFile(sb.ToString)
				
			Next		'Next result
		Next			'Next cohort item
	Next				'Next row in the job file
	
	Log("===========================")
	Log("  All done!")
	Log("===========================")
	ExitApplication2(0)
	Return r		'Just to keep the compliter happy...
End Sub

Private Sub responseParser ( strJson As String, listValues As List ) As List
	'Returns a list of maps.Each map containing the reuested values
	Dim returnList As List
	returnList.Initialize
	
	Dim parser As JSONParser
	parser.Initialize(strJson)
	Dim root As Map = parser.NextObject
	Dim results As List = root.Get("results")
	For Each colresults As Map In results
		Dim lastEditedStamp As Map = Main.utils.getMapFromMap(colresults,"lastEditedStamp")
		Dim userIdentifier As String = Main.utils.getStringFromMap(lastEditedStamp,"userIdentifier")
		Dim subject As Map = Main.utils.getMapFromMap(colresults,"subject")
		Dim id As Int = Main.utils.getIntFromMap(subject,"id")
		Dim subjectType As String = Main.utils.getStringFromMap(subject,"type")
		Dim createdStamp As Map = Main.utils.getMapFromMap(colresults,"createdStamp")
		Dim userIdentifier As String = Main.utils.getStringFromMap(createdStamp,"userIdentifier")
		Dim xType As String = Main.utils.getStringFromMap(colresults,"type")
		Dim temporalStatus As String = Main.utils.getStringFromMap(colresults,"temporalStatus")
		Dim roomDescription As String = Main.utils.getStringFromMap(colresults,"roomDescription")
		Dim floorDescription As String = Main.utils.getStringFromMap(colresults,"floorDescription")
		Dim map_location As Map = Main.utils.getMapFromMap(colresults,"location")
		Dim uprn As String = Main.utils.getStringFromMap(map_location,"uprn")
		Dim poBoxNumber As String = Main.utils.getStringFromMap(map_location,"poBoxNumber")
		Dim secondaryNameOrNumber As String = Main.utils.getStringFromMap(map_location,"secondaryNameOrNumber")
		Dim town As String = Main.utils.getStringFromMap(map_location,"town")
		Dim locality As String = Main.utils.getStringFromMap(map_location,"locality")
		Dim county As String = Main.utils.getStringFromMap(map_location,"county")
		Dim postcode As String = Main.utils.getStringFromMap(map_location,"postcode")
		Dim coordinates As Map = Main.utils.getMapFromMap(map_location,"coordinates")
		Dim latitude As Double = Main.utils.getDoubleFromMap(coordinates,"latitude")
		Dim longitude As Double = Main.utils.getDoubleFromMap(coordinates,"longitude")
		Dim location_type As String = Main.utils.getStringFromMap(map_location,"_type")
		Dim postTown As String = Main.utils.getStringFromMap(map_location,"postTown")
		Dim unverifiedUprn As String = Main.utils.getStringFromMap(map_location,"unverifiedUprn")
		Dim countyCode As String = Main.utils.getStringFromMap(map_location,"countyCode")
		Dim primaryNameOrNumber As String = Main.utils.getStringFromMap(map_location,"primaryNameOrNumber")
		Dim street As String = Main.utils.getStringFromMap(map_location,"street")
		Dim countryCode As String = Main.utils.getStringFromMap(map_location,"countryCode")
		Dim countrySubdivisionCode As String = Main.utils.getStringFromMap(map_location,"countrySubdivisionCode")
'		Dim location As String = Main.utils.getStringFromMap(map_location,"location")
		Dim id As Int = Main.utils.getIntFromMap(map_location,"id")
'		Dim personId As Int = Main.utils.getIntFromMap(colresults,"personId")
		Dim id As Int = Main.utils.getIntFromMap(colresults,"id")
'		Dim startDate As Map = Main.utils.getMapFromMap(colresults,"startDate")
'		Dim calculatedDate As String = Main.utils.getStringFromMap(startDate,"calculatedDate")

		Dim returnMap As Map
		returnMap.Initialize
		
		Dim x As Int
		For x = 0 To listValues.Size -1
			Select listValues.Get(x)
				Case "id" : returnMap.Put(listValues.Get(x), id)
				Case "primaryNameOrNumber" : returnMap.Put(listValues.Get(x), primaryNameOrNumber)
				Case "street" : returnMap.Put(listValues.Get(x), street)
				Case "locality" : returnMap.Put(listValues.Get(x), locality)
				Case "town" : returnMap.Put(listValues.Get(x), town)
				Case "county" : returnMap.Put(listValues.Get(x), county)
				Case "postcode" : returnMap.Put(listValues.Get(x), postcode)
				Case "uprn" : returnMap.Put(listValues.Get(x), uprn)
				Case "postTown" : returnMap.Put(listValues.Get(x), postTown)
				Case "type" : returnMap.Put(listValues.Get(x), xType)
				Case "temporalStatus" : returnMap.Put(listValues.Get(x), temporalStatus)
'				Case "location" : returnMap.Put(listValues.Get(x), location)
				Case "subjectType" : returnMap.Put(listValues.Get(x), subjectType)
				Case "userIdentifier" : returnMap.Put(listValues.Get(x), userIdentifier)
				Case "roomDescription" : returnMap.Put(listValues.Get(x), roomDescription)
				Case "floorDescription" : returnMap.Put(listValues.Get(x), floorDescription)
				Case "poBoxNumber" : returnMap.Put(listValues.Get(x), poBoxNumber)
				Case "secondaryNameOrNumber" : returnMap.Put(listValues.Get(x), secondaryNameOrNumber)
				Case "location_type" : returnMap.Put(listValues.Get(x), location_type)
				Case "countyCode" : returnMap.Put(listValues.Get(x), countyCode)
				Case "countryCode" : returnMap.Put(listValues.Get(x), countryCode)
				Case "countrySubdivisionCode" : returnMap.Put(listValues.Get(x), countrySubdivisionCode)
'				Case "calculatedDate" : returnMap.Put(listValues.Get(x), calculatedDate)
'				Case "preferred" : returnMap.Put(listValues.Get(x), preferred)
				Case "latitude" : returnMap.Put(listValues.Get(x), latitude)
				Case "longitude" : returnMap.Put(listValues.Get(x), longitude)
				Case "unverifiedUprn" : returnMap.Put(listValues.Get(x), unverifiedUprn)
				Case Else : returnMap.Put(listValues.Get(x), "UNKNOWN VALUE")
			End Select
			
		Next			'Next value we are asked to return
		returnList.Add(returnMap)
		
	Next		'Next result

	Return returnList
End Sub




