﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/classification/?status=ACTIVE&pageNumber=1&pageSize=-1&%5Bid%5D=asc"		'Must start with "/"
	Private header() As String = Array As String("Accept","application/vnd.olmgroup-usp.classification.ClassificationWithClassificationGroupsList+json;requestedAccess=WRITE")


	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ()
	
	'These are Optional items that can appear in the job file and are sent to the API
	Private listValuesToRetrieve As List		'Values for the API responses to return in a new file
	
	Private outputFileIsCreated As Boolean
	Private mapArgumentDefaults As Map
														
	Private jobFilePath As String
	Private jobFileName As String

	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	'Output file details
	Private listOutputFileHdrs As List
	Private listOutputFileRows As List
	
	Private previousValuesToRetrieve As String
	

End Sub


'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)
	'Set up default arguments
	mapArgumentDefaults.Initialize
		
	listValuesToRetrieve.Initialize
	listOutputFileHdrs.Initialize
	listOutputFileRows.Initialize
	
	jobFilePath = fPath
	jobFileName = fName
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	
	outputFileIsCreated = False		'Indicates if we are appending to the file or not
	
End Sub

Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If mapArgumentDefaults.ContainsKey(listMandatoryArguments.Get(x)) = False Then
			'This mandatory argument doesn't have a default therefore it must exist in the CSV headers
			If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
				'it does not feature in the column list
				listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
			End If
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

Public Sub Execute As ResumableSub
	'Process the job file
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	
	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file (There will only be 1 row in this file
		Dim rw() As String = listJobFileRows.Get(rowNum)
		
			
		'Get the list of values to retrieve from responses
		idx = listJobFileHdrs.IndexOf("VALUES_TO_RETRIEVE")
		If idx <> -1 Then
			If rw(idx) = "" Then
				rw(idx) = previousValuesToRetrieve
			Else
				previousValuesToRetrieve = rw(idx)
			End If
			listValuesToRetrieve = Main.sf.Split(rw(idx),",")
		End If
			
		listOutputFileHdrs.AddAll(listValuesToRetrieve)
		'strip any empty names that came across
		For x = listOutputFileHdrs.Size -1 To 0 Step -1
			If listOutputFileHdrs.Get(x) = "" Then listOutputFileHdrs.RemoveAt(x)
		Next
			
		'Construct the API call
		Dim rc As RestCall
		rc.Initialize
			
		' --------------------------------------------------------
		'	A P I   C A L L
		' --------------------------------------------------------
						
		'Make the API call
		'=================
		Dim payload as String =""
		Wait For (rc.restGet(apiPath, header, payload, 1)) Complete

		Dim listResults As List
		listResults = responseParser(rc.strResponse, listValuesToRetrieve)
		
		'We now have a list of maps. Each map containing a set of requested return values
				
		If outputFileIsCreated = False Then
			'Start a new CSV results file
			If File.Exists(Main.gresultsPath, Main.gresultsName) = True Then
				If File.Delete(Main.gresultsPath, Main.gresultsName) = False Then
					Log("Unable to delete previous results file")
					ExitApplication2(2)
				End If
			End If
			Dim sb As StringBuilder
			sb.Initialize
			For x = 0 To listOutputFileHdrs.Size -1
				If sb.Length > 1 Then sb.Append(",")
				sb.Append(QUOTE).Append(listOutputFileHdrs.Get(x)).Append(QUOTE)
			Next
			Main.utils.addToOutputFile(sb.ToString)
			outputFileIsCreated = True
		End If
					
		
		'Results now contained  (each one a map containing required values)
		For x = 0 To listResults.Size -1	'For each result

			Dim mapJ As Map = listResults.Get(x)
			
			Dim col As Int
			Dim sb As StringBuilder
			sb.Initialize

			For col = 0 To listOutputFileHdrs.Size -1	'For each column
				Dim strCol As String
				If mapJ.ContainsKey(listOutputFileHdrs.Get(col)) Then
					'This column comes from the API response
					
					'Check if this column is to be excluded if empty
					strCol = mapJ.Get(listOutputFileHdrs.Get(col))
					strCol = Main.utils.deNull(strCol)
				End If
				
				If sb.Length > 0 Then sb.Append(",")
				sb.Append(QUOTE).Append(strCol).Append(QUOTE)
			Next		'Next column
			
			Main.utils.addToOutputFile(sb.ToString)
			
		Next		'Next result
	Next				'Next row in the job file
	
	Log("===========================")
	Log("  All done!")
	Log("===========================")
	ExitApplication2(0)
	Return r		'Just to keep the compliter happy...
End Sub

Private Sub responseParser ( strJson As String, listValues As List ) As List
	'Returns a list of maps.Each map containing the reuested values
	Dim returnList As List
	returnList.Initialize
	
	
	Dim parser As JSONParser
	parser.Initialize(strJson)
	Dim root As Map = parser.NextObject
	Dim results As List = root.Get("results")
	For Each colresults As Map In results
		Dim code As String = colresults.Get("code")
		Dim classification_type As String = colresults.Get("_type")
		Dim parentGroupId As Int = colresults.Get("parentGroupId")
		Dim classificationGroupsList As List = colresults.Get("classificationGroupsList")
		Dim classification_group As String
		For Each colclassificationGroupsList As String In classificationGroupsList
			classification_group = classification_group & ">" & colclassificationGroupsList
		Next
		Dim name As String = colresults.Get("name")
		Dim id As Int = colresults.Get("id")
		Dim status As String = colresults.Get("status")
		Dim returnMap As Map
		returnMap.Initialize
		
		Dim x As Int
		For x = 0 To listValues.Size -1
		Select listValues.Get(x)
			Case "code" : returnMap.Put(listValues.Get(x), code)
			Case "classification_type" : returnMap.Put(listValues.Get(x), classification_type)
			Case "parentGroupId" : returnMap.Put(listValues.Get(x), parentGroupId)
			Case "classification_group" : returnMap.Put(listValues.Get(x), classification_group)
			Case "name" : returnMap.Put(listValues.Get(x), name)
			Case "id" : returnMap.Put(listValues.Get(x), id)
			Case "status" : returnMap.Put(listValues.Get(x), status)
			Case Else : returnMap.Put(listValues.Get(x), "UNKNOWN VALUE")
		End Select
			
		Next			'Next value we are asked to return
		returnList.Add(returnMap)
	Next
	
	Return returnList
End Sub
	
	
	
	
	
	
	
	
	
	
	
	
