﻿B4J=true
Group=Default Group
ModulesStructureVersion=1
Type=Class
Version=6.3
@EndOfDesignText@
Sub Class_Globals
	Private apiPath As String = "/eclipse/rest/person"		'Must start with "/"
	Private header() As String = Array As String()


	Private requiredFixedArgument As String = "s=%5B%5D"
	'These are Mandatory items that must occur in the job file
	Private listMandatoryArguments As List = Array As String ( 	"pageNumber", _
																"pageSize", _
																"useSoundex", _
																"escapeSpecialCharacters", _
																"appendWildcard", _
																"minAge", _
																"maxAge", _
																"lifeState" )
																													
	'These are Optional items that can appear in the job file and are sent to the API																										
	Private listOptionalArguments As List = Array As String ( )
	Private listValuesToRetrieve As List		'Values for the API responses to return in a new file	
	Private listValuesToCarryOver As List		'Values used to map original fields to retrieved values	
	Private listExcludeIfEmpty As List
	Private listExcludeIfNotEmpty As List
	
	Private outputFileIsCreated As Boolean	
	Private mapArgumentDefaults As Map
														
	Private jobFilePath As String
	Private jobFileName As String

	Private listJobFileHdrs As List
	Private listJobFileRows As List
	
	'Output file details
	Private listOutputFileHdrs As List
	Private listOutputFileRows As List
	
	Private previousValuesToRetrieve As String
	Private previousValuesToCarryOver As String
	Private previousExcludeIfEmpty As String
	Private previousExcludeIfNotEmpty As String
	
	
	Private gPageNumber As Int
	Private bRepeatCall As Boolean
	Private gRowsReceived As Long
	
	Private gTotalSize As Long				'Captures the anticipated total size of the possible data set
		
End Sub
'Detail here the mandatory and optional columns expected in the job csv file

'Initializes the object. You can add parameters to this method if needed.
Public Sub Initialize( fPath As String, fName As String)	
	'Set up default arguments
	mapArgumentDefaults.Initialize
	mapArgumentDefaults.Put("useSoundex","false")
	mapArgumentDefaults.Put("escapeSpecialCharacters","true")
	mapArgumentDefaults.Put("appendWildcard","true")

	mapArgumentDefaults.Put("pageSize","10")
		
	listValuesToCarryOver.Initialize
	listValuesToRetrieve.Initialize
	listOutputFileHdrs.Initialize
	listOutputFileRows.Initialize
	listExcludeIfEmpty.Initialize
	listExcludeIfNotEmpty.Initialize
	
	jobFilePath = fPath
	jobFileName = fName
	
	listJobFileHdrs.Initialize
	listJobFileRows.Initialize
	
	outputFileIsCreated = False		'Indicates if we are appending to the file or not
	
End Sub

Public Sub Validate As returnDetails
	'This method also reads the file into the listJobFileRows and listJobFileHdrs lists ready for processing
	Dim r As returnDetails
	r.Initialize
	'Check the job file contains the necessary mandatory fields
	Try
		listJobFileRows = Main.su.LoadCSV2(jobFilePath, jobFileName, ",", listJobFileHdrs)
		Main.logResult("Job file contains " & listJobFileRows.Size & " rows of " & listJobFileHdrs.Size & " columns")
	Catch
		Log(LastException)
		r.success = False
		r.value = -100
		r.description = "Failed to load the Job file. CSV Error"
		r.fatal = True
		Return r
	End Try
	
	'Check the required columns exist
	Dim x As Int
	Dim listValidationErrors As List
	listValidationErrors.Initialize
	For x = 0 To listMandatoryArguments.Size -1
		If mapArgumentDefaults.ContainsKey(listMandatoryArguments.Get(x)) = False Then
			'This mandatory argument doesn't have a default therefore it must exist in the CSV headers
			If listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x)) = -1 Then
				'it does not feature in the column list
				listValidationErrors.Add("ERROR - Mandatory column/argument " & listMandatoryArguments.Get(x) & " not found in job file")
			End If
		End If
	Next
	
	'Check and report errors in the above required columns check
	If listValidationErrors.Size > 0 Then
		For Each str As String In listMandatoryArguments
			Main.logResult(str)
		Next
		r.success = False
		r.value = -100
		r.description = "One or more mandatory columns/arguments missing"
		r.fatal = True
		Return r
	End If
	
	r.success = True
	r.value = 0
	r.description = "Validation passed"
	r.fatal = False
	Return r
End Sub

Public Sub Execute As ResumableSub 
	'Process the job file
	Dim rowNum As Int
	Dim x As Int
	Dim idx As Int

	Dim r As returnDetails
	r.Initialize
	
	For rowNum = 0 To listJobFileRows.Size - 1			'For each row in the job file
		gRowsReceived = 0		'Count of the number of rows of data received for each row in the job file
		gPageNumber = 1			'Default starting page (Can be overuled in the job file though)
		
		Dim rw() As String = listJobFileRows.Get(rowNum)
		
		'Construct a payload MAP containing values from the job file, the default list and the special values list
		Dim mapCall As Map		'Contains all of the elements for the API call as Key,Value pairs
		mapCall.Initialize
		

		'Grab the mandatory items
		For x = 0 To listMandatoryArguments.Size -1
			idx = listJobFileHdrs.IndexOf(listMandatoryArguments.Get(x))
			If idx <> -1 Then
				'Add this to the argument list
				Dim strVal As String = rw(idx)
				If strVal = "" Then
					'No value provided in job file, its it one of our defaults - substitute it
					If mapArgumentDefaults.ContainsKey(listMandatoryArguments.Get(x)) Then
						strVal = mapArgumentDefaults.Get(listMandatoryArguments.Get(x))
					End If
				End If
				mapCall.Put(listMandatoryArguments.Get(x), strVal)
			End If
		Next		'Next mandatory argument
		
		'Grab the non-mandatory (Optional) argument values from the job file
		For x = 0 To listOptionalArguments.Size -1
			idx = listJobFileHdrs.IndexOf(listOptionalArguments.Get(x))
			If idx <> -1 Then
				'This optional argument appears in the job file - include it
				If mapCall.ContainsKey(listOptionalArguments.Get(x)) = False Then
					mapCall.Put(listOptionalArguments.Get(x), rw(idx))
				End If
			End If
		Next		'Next optional argument
				
		'Get the list of values to retrieve from responses
		idx = listJobFileHdrs.IndexOf("VALUES_TO_RETRIEVE")
		If idx <> -1 Then
			If rw(idx) = "" Then
				rw(idx) = previousValuesToRetrieve
			Else
				previousValuesToRetrieve = rw(idx)
			End If
			listValuesToRetrieve = Main.sf.Split(rw(idx),",")
		End If
			
		'Get the list of values to carry over
		idx = listJobFileHdrs.IndexOf("VALUES_TO_CARRY_OVER")
		If idx <> -1 Then
			If rw(idx) = "" Then
				rw(idx) = previousValuesToCarryOver
			Else
				previousValuesToCarryOver = rw(idx)
			End If
			listValuesToCarryOver = Main.sf.Split(rw(idx),",")
		End If
			
		'Check the carry over values actually exist in the job file
		If listValuesToCarryOver.Size > 0 Then
			For x = (listValuesToCarryOver.Size -1) To 0 Step -1
				If listValuesToCarryOver.Get(x) <> "" Then
					idx = listJobFileHdrs.IndexOf(listValuesToCarryOver.Get(x))
					If idx = -1 Then
						'Doesn't exist - lose it
						listValuesToCarryOver.RemoveAt(x)
					End If
				End If
			Next
		End If
			
		'Get the list of values to check to exclude from output if empty
		idx = listJobFileHdrs.IndexOf("EXCLUDE_IF_EMPTY")
		If idx <> -1 Then
			If rw(idx) = "" Then
				rw(idx) = previousExcludeIfEmpty
			Else
				previousExcludeIfEmpty = rw(idx)
			End If
			listExcludeIfEmpty = Main.sf.Split(rw(idx),",")
		End If
			
		'Get the list of values to check to exclude from output if NOT empty
		idx = listJobFileHdrs.IndexOf("EXCLUDE_IF_NOT_EMPTY")
		If idx <> -1 Then
			If rw(idx) = "" Then
				rw(idx) = previousExcludeIfNotEmpty
			Else
				previousExcludeIfNotEmpty = rw(idx)
			End If
			listExcludeIfNotEmpty = Main.sf.Split(rw(idx),",")
		End If

		
		'Create header columns for the output file
		listOutputFileHdrs.AddAll(listValuesToCarryOver)
		listOutputFileHdrs.AddAll(listValuesToRetrieve)
		'strip any empty names that came across
		For x = listOutputFileHdrs.Size -1 To 0 Step -1
			If listOutputFileHdrs.Get(x) = "" Then listOutputFileHdrs.RemoveAt(x)
		Next
		
		'Construct the API call
		Dim rc As RestCall
		rc.Initialize
			
		bRepeatCall = True
		' --------------------------------------------------------
		'	A P I   C A L L   L O O P
		' --------------------------------------------------------	
		Do While bRepeatCall
			'Construct the payload for the API call
			'Set the page number (for iterative calls)
			If mapCall.ContainsKey("pageNumber") Then
				mapCall.Put("pageNumber", gPageNumber)
			End If
			
			'Create a payload from the map
			Dim sb As StringBuilder
			sb.Initialize
			For x = 0 To mapCall.Size -1
				If x > 0 Then sb.Append("&")
				sb.Append(mapCall.GetKeyAt(x)).Append("=").Append(mapCall.GetValueAt(x))
			Next

			Dim payload As String
			payload = requiredFixedArgument & "&" & sb.ToString
						
			'Make the API call
			'=================
			Wait For (rc.restGet(apiPath, header, payload, gPageNumber)) Complete

			'Log("API response returned")
			Dim listResults As List
			listResults = responseParser(rc.strResponse, listValuesToRetrieve)
			
			'We now have a list of maps. Each map containing a set of requested return values
			
			gRowsReceived = gRowsReceived + listResults.Size			
			
			
			'Determine if we have all of the data yet
			If (gRowsReceived >= gTotalSize) Or _
				((gRowsReceived >= Main.gMaxRowsReturn) And Main.gMaxRowsReturn > 0) Then
				'All rows have now been received
				bRepeatCall = False		'No more API calls
			Else
				gPageNumber = gPageNumber + 1
			End If
			
			If outputFileIsCreated = False Then
				'Start a new CSV results file
				If File.Exists(Main.gresultsPath, Main.gresultsName) = True Then
					If File.Delete(Main.gresultsPath, Main.gresultsName) = False Then
						Log("Unable to delete previous results file")
						ExitApplication2(2)
					End If
				End If
				Dim sb As StringBuilder
				sb.Initialize
				For x = 0 To listOutputFileHdrs.Size -1
					If sb.Length > 1 Then sb.Append(",")
					sb.Append(QUOTE).Append(listOutputFileHdrs.Get(x)).Append(QUOTE)
				Next
				Main.utils.addToOutputFile(sb.ToString)
				outputFileIsCreated = True
			End If
			
			
			'Results now contained  (each one a map containing required values)
			For x = 0 To listResults.Size -1	'For each result

				Dim mapJ As Map = listResults.Get(x)
						
				Dim includeThisRow As Boolean = True
				Dim col As Int
				Dim sb As StringBuilder
				sb.Initialize

				For col = 0 To listOutputFileHdrs.Size -1	'For each column
					Dim strCol As String
					If mapJ.ContainsKey(listOutputFileHdrs.Get(col)) Then
						'This column comes from the API response
								
						'Check if this column is to be excluded if empty
						strCol = mapJ.Get(listOutputFileHdrs.Get(col))
						strCol = Main.utils.deNull(strCol)
						If listExcludeIfEmpty.IndexOf(listOutputFileHdrs.Get(col)) <> -1 Then
							'Exclude this column if empty
							If strCol = "" Then
								includeThisRow = False
								Exit
							End If
						End If
						'Check if this column is to be excluded if NOT empty
						If listExcludeIfNotEmpty.IndexOf(listOutputFileHdrs.Get(col)) <> -1 Then
							'Exclude this column if NOT empty
							If strCol <> "" Then
								includeThisRow = False
								Exit
							End If
						End If
					Else
						idx = listValuesToCarryOver.IndexOf(listOutputFileHdrs.Get(col))
						If idx <> -1 Then
							'This column is one of the values we are asked to carry across from the job file
							Dim idx2 As Int = listJobFileHdrs.IndexOf(listOutputFileHdrs.Get(col))
							If idx2 <> -1 Then
								strCol = rw(idx2)
							Else
								strCol = "WRONG VALUE_TO_CARRY_OVER"
							End If
						Else
							strCol = "WRONG COLUMN NAME"
						End If
					End If
							
					If sb.Length > 0 Then sb.Append(",")
					sb.Append(QUOTE).Append(strCol).Append(QUOTE)
				Next		'Next column
						
				If includeThisRow Then Main.utils.addToOutputFile(sb.ToString)
						
			Next		'Next result
		Loop			'Still not all results received....
	Next				'Next row in the job file
			
	Log("===========================")	
	Log("  All done!")
	Log("===========================")

	ExitApplication2(0)
	Return r		'Just to keep the compliter happy...
End Sub


Private Sub responseParser ( strJson As String, listValues As List ) As List
'Returns a list of maps.Each map containing the reuested values
	Dim returnList As List
	returnList.Initialize

	Dim parser As JSONParser
	parser.Initialize(strJson)

	Dim root As Map = parser.NextObject

	'Dim sortSelection As Map = root.Get("sortSelection")
	Dim pageNumber As Int = Main.utils.getintfrommap( root,"pageNumber")
	Dim totalSize As Int = Main.utils.getintfrommap( root,"totalSize")
	gTotalSize = totalSize		'Remember this value
	Dim offset As Int = Main.utils.getintfrommap( root,"offset")
	Dim pageSize As Int = Main.utils.getintfrommap( root,"pageSize")
	Dim results As List = Main.utils.getlistfrommap( root,"results")

	For Each colresults As Map In results
		Dim gender As String = Main.utils.getstringfrommap( colresults,"gender")
		Dim ethnicity As String = Main.utils.getstringfrommap( colresults,"ethnicity")

		Dim dueDate As String = Main.utils.getstringfrommap( colresults,"dueDate")
		Dim diedDateEstimated As String = Main.utils.getstringfrommap( colresults,"diedDateEstimated")
		Dim title As String = Main.utils.getstringfrommap( colresults,"title")
		Dim suffix As String = Main.utils.getstringfrommap( colresults,"suffix")
		Dim middleNames As String = Main.utils.getstringfrommap( colresults,"middleNames")
		Dim chiNumber As String = Main.utils.getstringfrommap( colresults,"chiNumber")
		Dim duplicated As String = Main.utils.getstringfrommap( colresults,"duplicated")
		Dim forename As String = Main.utils.getstringfrommap( colresults,"forename")
		Dim genderIdentity As String = Main.utils.getstringfrommap( colresults,"genderIdentity")
		Dim pictureId As String = Main.utils.getstringfrommap( colresults,"pictureId")
		Dim countryOfBirth As String = Main.utils.getstringfrommap( colresults,"countryOfBirth")
		Dim surname As String = Main.utils.getstringfrommap( colresults,"surname")
		Dim knownAs As String = Main.utils.getstringfrommap( colresults,"knownAs")
		Dim notCarriedToTermType As String = Main.utils.getstringfrommap( colresults,"notCarriedToTermType")
		Dim dateOfBirthEstimated As String = Main.utils.getstringfrommap( colresults,"dateOfBirthEstimated")
		Dim professionalTitle As String = Main.utils.getstringfrommap( colresults,"professionalTitle")
		Dim id As Int = Main.utils.getintfrommap( colresults,"id")
		Dim lifeState As String = Main.utils.getstringfrommap( colresults,"lifeState")
		Dim results_type As String = Main.utils.getstringfrommap( colresults,"_type")

		Dim dateOfBirth As Map = Main.utils.getmapfrommap( colresults,"dateOfBirth")
'		Dim calculatedDate As String = Main.utils.getstringfrommap( dateOfBirth,"calculatedDate")
		Dim dateOfBirth_type As String = Main.utils.getstringfrommap( dateOfBirth,"_type")
		Dim fuzzyDateMask As String = Main.utils.getstringfrommap( dateOfBirth,"fuzzyDateMask")

		Dim allowedRange As Map = Main.utils.getmapfrommap( dateOfBirth,"allowedRange")
		Dim lowerBoundDate As String = Main.utils.getstringfrommap( allowedRange,"lowerBoundDate")
		Dim upperBoundDate As String = Main.utils.getstringfrommap( allowedRange,"upperBoundDate")

		Dim diedDate As String = Main.utils.getstringfrommap( colresults,"diedDate")
		Dim personTypes As List = Main.utils.getlistfrommap( colresults,"personTypes")

		Dim personTypesList As String
		For Each colpersonTypes As String In personTypes
			If personTypesList <> "" Then personTypesList = personTypesList & ","
			personTypesList = personTypesList & colpersonTypes 
		Next

		Dim nhsNumber As String = Main.utils.getstringfrommap( colresults,"nhsNumber")
		Dim religion As String = Main.utils.getstringfrommap( colresults,"religion")
		'Dim objectVersion As Int = Main.utils.getintfrommap( colresults,"objectVersion")
		Dim sexualOrientation As String = Main.utils.getstringfrommap( colresults,"sexualOrientation")
		'Dim results_secured As String = Main.utils.getstringfrommap( colresults,"_secured")
		Dim organisationName As String = Main.utils.getstringfrommap( colresults,"organisationName")
		Dim alternateNames As String = Main.utils.getstringfrommap( colresults,"alternateNames")
		Dim name As String = Main.utils.getstringfrommap( colresults,"name")
		Dim personIdentifier As String = Main.utils.getstringfrommap( colresults,"personIdentifier")
		Dim genderIdentitySelfDefined As String = Main.utils.getstringfrommap( colresults,"genderIdentitySelfDefined")
		Dim age As Int = Main.utils.getintfrommap( colresults,"age")


		Dim returnMap As Map
		returnMap.Initialize

		Dim x As Int
		For x = 0 To listValues.Size -1
			Select listValues.Get(x)
				
				Case "pageNumber" : returnMap.Put(listValues.Get(x), pageNumber)
				Case "totalSize" : returnMap.Put(listValues.Get(x), totalSize)
				Case "offset" : returnMap.Put(listValues.Get(x), offset)
				Case "pageSize" : returnMap.Put(listValues.Get(x), pageSize)
				Case "gender" : returnMap.Put(listValues.Get(x), gender)
				Case "ethnicity" : returnMap.Put(listValues.Get(x), ethnicity)
				Case "dueDate" : returnMap.Put(listValues.Get(x), dueDate)
				Case "diedDateEstimated" : returnMap.Put(listValues.Get(x), diedDateEstimated)
				Case "title" : returnMap.Put(listValues.Get(x), title)
				Case "suffix" : returnMap.Put(listValues.Get(x), suffix)
				Case "middleNames" : returnMap.Put(listValues.Get(x), middleNames)
				Case "chiNumber" : returnMap.Put(listValues.Get(x), chiNumber)
				Case "duplicated" : returnMap.Put(listValues.Get(x), duplicated)
				Case "forename" : returnMap.Put(listValues.Get(x), forename)
				Case "genderIdentity" : returnMap.Put(listValues.Get(x), genderIdentity)
				Case "pictureId" : returnMap.Put(listValues.Get(x), pictureId)
				Case "countryOfBirth" : returnMap.Put(listValues.Get(x), countryOfBirth)
				Case "surname" : returnMap.Put(listValues.Get(x), surname)
				Case "knownAs" : returnMap.Put(listValues.Get(x), knownAs)
				Case "notCarriedToTermType" : returnMap.Put(listValues.Get(x), notCarriedToTermType)
				Case "dateOfBirthEstimated" : returnMap.Put(listValues.Get(x), dateOfBirthEstimated)
				Case "professionalTitle" : returnMap.Put(listValues.Get(x), professionalTitle)
				Case "id" : returnMap.Put(listValues.Get(x), id)
				Case "lifeState" : returnMap.Put(listValues.Get(x), lifeState)
				Case "results_type" : returnMap.Put(listValues.Get(x), results_type)
'				Case "calculatedDate" : returnMap.Put(listValues.Get(x), calculatedDate)
				Case "dateOfBirth_type" : returnMap.Put(listValues.Get(x), dateOfBirth_type)
				Case "fuzzyDateMask" : returnMap.Put(listValues.Get(x), fuzzyDateMask)
				Case "lowerBoundDate" : returnMap.Put(listValues.Get(x), lowerBoundDate)
				Case "upperBoundDate" : returnMap.Put(listValues.Get(x), upperBoundDate)
				Case "diedDate" : returnMap.Put(listValues.Get(x), diedDate)
				Case "personTypesList" : returnMap.Put(listValues.Get(x), personTypesList)
				Case "nhsNumber" : returnMap.Put(listValues.Get(x), nhsNumber)
				Case "religion" : returnMap.Put(listValues.Get(x), religion)
				Case "sexualOrientation" : returnMap.Put(listValues.Get(x), sexualOrientation)
				Case "organisationName" : returnMap.Put(listValues.Get(x), organisationName)
				Case "alternateNames" : returnMap.Put(listValues.Get(x), alternateNames)
				Case "name" : returnMap.Put(listValues.Get(x), name)
				Case "personIdentifier" : returnMap.Put(listValues.Get(x), personIdentifier)
				Case "genderIdentitySelfDefined" : returnMap.Put(listValues.Get(x), genderIdentitySelfDefined)
				Case "age" : returnMap.Put(listValues.Get(x), age)

				Case Else : returnMap.Put(listValues.Get(x), "UNKNOWN VALUE")
			End Select
			
		Next			'Next value we are asked to return
		returnList.Add(returnMap)
	Next
	
	Return returnList
End Sub

